from .runner import go
from .pen import Pen
from .pen import Font
from .data import Data
from .master import Master
from .actor import Actor
from .animation import animate

from pgzero import animation,  clock, loaders, music,  rect, runner, screen, constants, keyboard
from pgzero.clock import clock, tick, schedule
from pgzero.loaders import fonts, sounds, root, images
from pgzero.rect import Rect
from pgzero.screen import Screen
from pgzero.constants import mouse
from pgzero.keyboard import keys, keyboard



# 对pgzero里一些功能进行重新封装和说明，用于教学
from pgzero.animation import animate as ani


def animate(object, tween='linear', duration=1, on_finished=None, **targets):
    '''
    控制角色运动的功能，比如从一个地方运动到另外一个地方。\n  
    object：你要运动的角色(必选)。\n
    pos：终点坐标(必选)。
    tween：运动的方式，linear就是线性的意思，还有：accelerate、decelerate、accel_decel、end_elastic、start_elastic、both_elastic、bounce_end、bounce_start、bounce_start_end等参数可选，请大家自行尝试。\n
    duration：延迟的时间，单位为s。\n
    on_finished：结束后做的事。\n
    '''
    ani(object, tween='linear', duration=1, on_finished=None, **targets)



import os
import sys

import pygame
import pygame.draw
from pgzero import loaders, ptext
from pgzero.rect import Rect


def round_xy(pos):
    '四舍五入坐标'
    if isinstance(pos, tuple):
        x, y = pos
        return(round(x), round(y))
    else:
        return(round(pos))


class Font:
    '字体样式类'

    def __init__(self) -> None:
        '初始化字体样式'
        self.styles = {}
        path = os.path.join(loaders.root, 'fonts')
        if os.path.isdir(path):
            self.fonts = os.listdir(path)
            self.fontname = self.fonts[-1]  # 字体名字
        else:
            self.fonts = []
            self.fontname = None
        self.bold = None  # 加粗
        self.italic = None  # 斜体
        self.underline = None  # 下划线
        self.color = 'black'  # 字体颜色
        self.gcolor = None  # 渐变色
        self.ocolor = None  # 边框颜色
        self.scolor = None  # 阴影颜色
        self.align = 'left'  # 左对齐
        self.alpha = 1.0  # 不透明度
        self.angle = 0  # 旋转角度
        self.owidth = None  # 边框宽度
        self.shadow = (0.0, 0.0)  # 阴影，x方向和y方向
        self.fontsize = 30  # 字体大小
        self.sysfonts = pygame.font.get_fonts()  # 所有系统字体
        self.sysfontname = self.sysfonts[-1]  # 设置系统字体

    def __setattr__(self, name, value) -> None:
        '重写设置属性的方法'
        self.__dict__[name] = value
        if not name in ['sysfonts', 'styles', 'fonts']:
            self.styles[name] = value


class Pen:
    '画笔类，用于绘制图形或者写字'

    def __init__(self) -> None:
        '画笔类，用于绘制图形或者写字'
        self.mod = sys.modules['__main__']

    def dot(self, pos, radius, color):
        '绘制一个点，参数依次为：中心坐标，直径，颜色'
        pos = round_xy(pos)
        self.circle(pos, radius, color, 0)

    def line(self, start, end, color, border=1):
        '绘制一条线，参数依次为：起点、终点、颜色、宽度'
        start = round_xy(start)
        end = round_xy(end)
        surf = self.mod.screen.surface
        pygame.draw.line(surf, color, start, end, border)

    def circle(self, pos, radius, color, border=1):
        '''
        绘制圆圈或者圆环，参数依次为：圆心坐标、直径、颜色、圆环宽度\n
        border参数默认为1，会绘制宽度为1的圆圈\n
        border参数设置成0，绘制实心圆\n
        '''
        pos = round_xy(pos)
        surf = self.mod.screen.surface
        pygame.draw.circle(surf, color, pos, radius, border)

    def ellipse(self, pos, width, height, color, border=1):
        '''
        绘制椭圆，参数依次为：中心坐标，宽度、高度、颜色、边框宽度\n
        border参数默认为1，会绘制宽度为1的椭圆\n
        border参数设置成0，绘制实心椭圆\n
        '''
        pos = round_xy(pos)
        surf = self.mod.screen.surface
        x = pos[0]-width/2
        y = pos[1]-height/2
        rect = Rect(x, y, width, height)
        pygame.draw.ellipse(surf, color, rect, border)

    def rect(self, pos, width, height, color, border=1, radius=0):
        '''
        绘制长方形或者正方形，参数以此为：中心坐标、宽度、高度、颜色、边框宽度、圆角半径\n
        边框宽度、圆角半径默认为1和0,会绘制一个边框为1没有圆角的方形\n
        '''
        pos = round_xy(pos)
        surf = self.mod.screen.surface
        x = pos[0]-width/2
        y = pos[1]-height/2
        rect = Rect(x, y, width, height)
        pygame.draw.rect(surf, color, rect, border, radius)

    def text(self, text, font=None, *args, **kwargs):
        '''
        写字(仅文本模式)\n  
        text参数是你要写的文字\n
        font是字体，实例化的Font类，如果不传递就会使用默认样式\n
        位置参数也必须传递，传递的时候参考Actor类的传递方式\n
        其他相关参数也可以手动传递
        '''
        # FIXME: expose ptext parameters, for autocompletion and autodoc
        if not font:
            font = Font()
        surf = self.mod.screen.surface
        kwargs.update(font.styles)
        kwargs['text'] = text
        ptext.draw(surf=surf, *args, **kwargs)

    def textbox(self, text, font=None, *args, **kwargs):
        '''
        写字(文本框模式)\n
        text参数是你要写的文字\n
        font是字体，实例化的Font类，如果不传递就会使用默认样式\n
        位置参数也必须传递，传递的时候参考Actor类的传递方式\n
        其他相关参数也可以手动传递
        '''
        # FIXME: expose ptext parameters, for autocompletion and autodoc
        if not font:
            font = Font()
        surf = self.mod.screen.surface
        kwargs.update(font.styles)
        kwargs['text'] = text
        ptext.drawbox(surf=surf, *args, **kwargs)



import datetime

from cpgzh import Rect
from cpgzh import Screen
from cpgzh import mouse
from cpgzh import keys
from cpgzh import Actor
from cpgzh import Master
from cpgzh import Font
from cpgzh import Pen
from cpgzh import go

font = Font()
font1 = Font()
font2 = Font()
font1.color = 'red'
font2.color = 'blue'
font2.bold = True
font2.angle = 180

screen: Screen
WIDTH = 800
HEIGHT = 800
rect = Rect(0, 0, WIDTH, HEIGHT)
jisi = Actor('大祭司_人偶-待机-000.png', center=(400, 400))
monv = Actor('恶魔女_人偶-待机-000.png', center=(800, 400))
monv.images = [f'恶魔女_人偶-待机-{i:0>3}.png' for i in range(30)]

master = Master()
pen = Pen()


def flushImages():
    '刷新造型列表'
    now = datetime.datetime.now()
    print(f'{now.time()} 刷新造型列表')
    jisi.images = [f'大祭司_人偶-待机-{i:0>3}.png' for i in range(30)]


def update():
    '更新数据'
    jisi.run_tasks()  # 执行计划任务
    monv.run_tasks()
    if jisi.collide_pixel(monv):
        print(f'碰撞点坐标：{jisi.collide_pixel(monv)}')


def draw():
    '绘制角色'
    screen.clear()
    screen.draw.filled_rect(rect, 'white')
    jisi.draw()
    monv.draw()
    if 'text' in dir(master.data):  # 如果data中有text属性
        screen.draw.text(
            str(master.data.text),
            pos=(100, 100),
            color='red',
            fontsize=40,
        )
    # 绘制一个点
    pen.dot((300, 50), 50, '#ffff00')
    # 绘制一条线
    pen.line((250, 50), (350, 50), '#00ffff', 20)
    # 绘制圆或者圆环
    pen.circle((100, 200), 100, '#ff0000', 20)
    # 绘制椭圆或者椭圆环
    pen.ellipse((300, 200), 200, 100, '#00ff00', 20)
    # 绘制方块或者方块环
    pen.rect((500, 200), 200, 200, '#0000ff', 10, 30)
    # font样式写字
    pen.text('haha', topleft=(200, 400))
    # font1样式写字
    pen.text('haha', font=font1, center=(200, 450))
    # font2样式写字
    pen.text('haha', font=font2, center=(200, 500))


def on_mouse_down(pos, button):
    '当鼠标按下'
    print(button)
    if button == mouse.LEFT:  # 按下左键刷新造型列表
        jisi.images = jisi.images = [jisi.image]
    else:  # 按下右键停止切换造型并延迟开启
        yesOrNo = master.yes_no('是否切换造型？')
        if yesOrNo:
            jisi.create_delay_tasks(flushImages, 5, 3)  # 延迟5s重置造型列表，循环3次


def on_mouse_move(pos):
    '当鼠标移动时'
    monv.x, monv.y = pos


def on_key_down(key):
    '当键盘按下'
    # 设置全屏化
    if key == keys.A:
        master.set_fullscreen()
    # 设置窗口化
    elif key == keys.B:
        master.set_windowed()
    # 切换全屏和窗口化
    elif key == keys.C:
        master.toggle_fullscreen()
    # 隐藏鼠标
    elif key == keys.D:
        master.hide_mouse()
    # 显示鼠标
    elif key == keys.E:
        master.show_mouse()
    # 输入文本
    elif key == keys.F:
        text = master.input('请输入一个名字:')
        print(text)
    # 选择文件
    elif key == keys.G:
        text = master.select_file('请选择一个文件：')
        print(text)
    # 保存文件
    elif key == keys.H:
        text = master.select_file_save('请选择存档保存位置：')
        print(text)
    # 选择文件夹
    elif key == keys.I:
        text = master.select_dir('请选择一个文件夹:')
        print(text)
    # 是否选择框
    elif key == keys.J:
        text = master.yes_no('是否选择女装？')
        print(text)
    # 保存master的数据
    elif key == keys.K:
        master.save_data()
    # 手动加载存储的数据
    elif key == keys.L:
        #master.data.text = 'test'
        master.data_path = '测试.dat'
        master.load_data()
    # 删除保存的数据
    elif key == keys.M:
        master.del_data()
    # 设置动画的帧率
    elif key == keys.N:
        fps = master.input('请输入动画切换的速度(每s切换多少次)：')
        jisi.animate_fps = int(fps)
    # 切换造型是否切换
    elif key == keys.O:
        jisi.toggle_animate()
    # 提示信息
    elif key == keys.P:
        master.msg('提示：\n你已经换好女装了')
    # 警告信息
    elif key == keys.Q:
        master.warning('警告:\n今天的女装不太完美')
    # 错误信息
    elif key == keys.R:
        master.error('错误：\n你还没换好女装')
    # 游戏开始
    elif key == keys.S:
        print(master.data.start())
    # 获取游戏运行时常
    elif key == keys.T:
        print(master.data.get_time())


go()
